Attribute VB_Name = "Module1"
Option Explicit
Enum ENUM_FOLDER_TYPE
     ENUM_API
     ENUM_VB
     ENUM_FSO
     ENUM_SHELL
End Enum
Const MAX_PATH = 260
Const INVALID_HANDLE_VALUE = -1
Const FILE_ATTRIBUTE_DIRECTORY = &H10

Type FILETIME
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

Type WIN32_FIND_DATA
  dwFileAttributes As Long
  ftCreationTime As FILETIME
  ftLastAccessTime As FILETIME
  ftLastWriteTime As FILETIME
  nFileSizeHigh As Long
  nFileSizeLow As Long
  dwReserved0 As Long
  dwReserved1 As Long
  cFileName As String * MAX_PATH
  cAlternate As String * 14
End Type

Type FOLDER_INFO
  DirSize As Currency
  FilesCount As Long
  DirsCount As Long
End Type

Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFindFile As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Public sFiles() As String

Public Function GetFolderInfo(sFolderPath As String, EnumType As ENUM_FOLDER_TYPE) As FOLDER_INFO
  If Right$(sFolderPath, 1) <> "\" Then sFolderPath = sFolderPath & "\"
  Dim lFileNum As Long, lDirNum As Long
  Dim curSize As Currency
  Select Case EnumType
      Case ENUM_API
           Call EnumFolderAPI(sFolderPath, lDirNum, lFileNum, curSize)
      Case ENUM_VB
           Call EnumFolderVB(sFolderPath, lDirNum, lFileNum, curSize)
      Case ENUM_FSO
           Call EnumFolderFSO(sFolderPath, lDirNum, lFileNum, curSize)
      Case ENUM_SHELL
           Call EnumFolderShell(sFolderPath, lDirNum, lFileNum, curSize)
  End Select
  GetFolderInfo.DirSize = curSize
  GetFolderInfo.FilesCount = lFileNum
  GetFolderInfo.DirsCount = lDirNum
End Function

'=====Get folder tree using ShellAutomation===========
Private Sub EnumFolderShell(sFolder As String, lDirNum As Long, lFileNum As Long, curSize As Currency)
   Dim oShell As Object
   Dim fld As Object, fldItem As Object
   Dim n As Integer
   Set oShell = CreateObject("Shell.Application")
   Set fld = oShell.NameSpace(sFolder)
   For Each fldItem In fld.Items
       If fldItem.IsFolder Then
          lDirNum = lDirNum + 1
          Call EnumFolderShell(fldItem.Path, lDirNum, lFileNum, curSize)
       Else
          lFileNum = lFileNum + 1
          n = UBound(sFiles) + 1
          ReDim Preserve sFiles(n)
          sFiles(n) = fldItem.Path
       End If
   Next
   Set fldItem = Nothing
   Set fld = Nothing
   Set oShell = Nothing
End Sub
'===========Get folder tree using FSO=================
Private Sub EnumFolderFSO(sFolder As String, lDirNum As Long, lFileNum As Long, curSize As Currency)
  Dim FSO  As Object, objFolder As Object
  Dim fld As Object, fyl As Object
  Dim n As Integer
  Set FSO = CreateObject("Scripting.FileSystemObject")
  On Error Resume Next
  Set objFolder = FSO.GetFolder(sFolder)
  For Each fyl In objFolder.Files
      curSize = curSize + fyl.Size
      lFileNum = lFileNum + 1
      n = UBound(sFiles) + 1
      ReDim Preserve sFiles(n)
      sFiles(n) = fyl.Path
  Next fyl
  If objFolder.SubFolders.Count Then
     For Each fld In objFolder.SubFolders
        lDirNum = lDirNum + 1
        EnumFolderFSO fld.Path, lDirNum, lFileNum, curSize
     Next fld
  End If
  Set fld = Nothing
  Set objFolder = Nothing
  Set FSO = Nothing
End Sub

'===========Get folder tree using VB =================
'VB Dir function can not be use for recursion, so some work around
Private Sub EnumFolderVB(sFolder As String, lDirNum As Long, lFileNum As Long, curSize As Currency)
   Dim FolderQueue As New Collection
   On Error Resume Next
   FolderQueue.Add sFolder
   Call DoEnumVB(FolderQueue, lDirNum, lFileNum, curSize)
   FolderQueue.Remove 1
   Do While FolderQueue.Count > 0
      Call DoEnumVB(FolderQueue, lDirNum, lFileNum, curSize)
      FolderQueue.Remove 1
      DoEvents
    Loop
End Sub

Private Sub DoEnumVB(FolderQueue As Collection, lDirNum As Long, lFileNum As Long, curSize As Currency)
   Dim sTemp As String, n As Integer, sFolder As String
   sFolder = FolderQueue.Item(1)
   sTemp = Dir$(sFolder & "*.*", vbNormal + vbArchive + vbDirectory + vbHidden + vbReadOnly + vbSystem)
   Do While sTemp <> ""
      If (GetAttr(sFolder & sTemp) And vbDirectory) = vbDirectory Then
         If sTemp <> "." And sTemp <> ".." Then
            If Right$(sTemp, 1) <> "\" Then sTemp = sTemp & "\"
            lDirNum = lDirNum + 1
            FolderQueue.Add sFolder & sTemp
         End If
      Else
         lFileNum = lFileNum + 1
         curSize = curSize + FileLen(sFolder & sTemp)
         n = UBound(sFiles) + 1
         ReDim Preserve sFiles(n)
         sFiles(n) = sFolder & sTemp
      End If
      sTemp = Dir$
   Loop
End Sub

'===========Get folder tree using API=================
Private Sub EnumFolderAPI(sFolder As String, lDirNum As Long, lFileNum As Long, curSize As Currency)
  Dim sTemp As String
  Dim lRet As Long, WFD As WIN32_FIND_DATA
  Dim hFile As Long, n As Integer
  hFile = FindFirstFile(sFolder & "*.*", WFD)
  If hFile = INVALID_HANDLE_VALUE Then Exit Sub
  sTemp = TrimNulls(WFD.cFileName)
  Do While sTemp <> ""
     If (WFD.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = FILE_ATTRIBUTE_DIRECTORY Then
        If sTemp <> "." And sTemp <> ".." Then
           If Right$(sTemp, 1) <> "\" Then sTemp = sTemp & "\"
           lDirNum = lDirNum + 1
           Call EnumFolderAPI(sFolder & sTemp, lDirNum, lFileNum, curSize)
        End If
     Else
        curSize = curSize + WFD.nFileSizeLow
        lFileNum = lFileNum + 1
        n = UBound(sFiles) + 1
        ReDim Preserve sFiles(n)
        sFiles(n) = sFolder & TrimNulls(WFD.cFileName)
     End If
     lRet = FindNextFile(hFile, WFD)
     sTemp = ""
     If lRet <> 0 Then sTemp = TrimNulls(WFD.cFileName)
  Loop
  lRet = FindClose(hFile)
End Sub

Private Function TrimNulls(sTemp As String) As String
  Dim l As Long
  l = InStr(1, sTemp, Chr(0))
  If l = 1 Then
     TrimNulls = ""
  ElseIf l > 0 Then
     TrimNulls = Left$(sTemp, l - 1)
  Else
     TrimNulls = sTemp
  End If
End Function

